import "bootstrap.css";
import "@grapecity/wijmo.styles/wijmo.css";
import "./styles.css";
import { Component, enableProdMode, NgModule } from "@angular/core";
import { platformBrowserDynamic } from "@angular/platform-browser-dynamic";
import { BrowserModule } from "@angular/platform-browser";
import {
  WjGridModule,
  WjFlexGridCellTemplate
} from "@grapecity/wijmo.angular2.grid";
import * as wjcCore from "@grapecity/wijmo";
import * as wjcGrid from "@grapecity/wijmo.grid";
import * as wjcInput from "@grapecity/wijmo.input";
import { CustomGridEditor } from "./app.customEditor";

@Component({
  selector: "app-component",
  templateUrl: "src/app.component.html"
})
export class AppComponent {
  data: wjcCore.CollectionView;
  countries = [
    { id: 0, name: "US" },
    { id: 1, name: "Germany" },
    { id: 2, name: "UK" },
    { id: 3, name: "Japan" },
    { id: 4, name: "Italy" },
    { id: 5, name: "Greece" }
  ];
  cities = [
    { id: 0, country: 0, name: "Washington" },
    { id: 1, country: 0, name: "Miami" },
    { id: 2, country: 0, name: "Seattle" },
    { id: 3, country: 1, name: "Bonn" },
    { id: 4, country: 1, name: "Munich" },
    { id: 5, country: 1, name: "Berlin" },
    { id: 6, country: 2, name: "London" },
    { id: 7, country: 2, name: "Oxford" },
    { id: 8, country: 2, name: "Manchester" },
    { id: 9, country: 3, name: "Tokyo" },
    { id: 10, country: 3, name: "Sendai" },
    { id: 11, country: 3, name: "Kobe" },
    { id: 12, country: 4, name: "Rome" },
    { id: 13, country: 4, name: "Florence" },
    { id: 14, country: 4, name: "Milan" },
    { id: 15, country: 5, name: "Athens" },
    { id: 16, country: 5, name: "Santorini" },
    { id: 17, country: 5, name: "Thebes" }
  ];
  countriesMap = new wjcGrid.DataMap(this.countries, "id", "name");
  citiesMap = new wjcGrid.DataMap(this.cities, "id", "name");
  // DataSvc will be passed by derived classes
  constructor() {
    this.data = this._getData();
  }

  initializeGrid(flex: wjcGrid.FlexGrid) {
    // add custom editors to the grid
    new CustomGridEditor(flex, "date", wjcInput.InputDate, {
      format: "d"
    });
    new CustomGridEditor(flex, "time", wjcInput.InputTime, {
      format: "t",
      min: new Date(2000, 1, 1, 7, 0),
      max: new Date(2000, 1, 1, 22, 0),
      step: 30
    });
    let countryEditor = new CustomGridEditor(
      flex,
      "country",
      wjcInput.ComboBox,
      {
        itemsSource: this.countries,
        displayMemberPath: "name",
        selectedValuePath: "id"
      }
    );
    new CustomGridEditor(flex, "amount", wjcInput.InputNumber, {
      format: "n2",
      step: 10
    });
    // create an editor based on a ComboBox
    let multiColumnEditor = new CustomGridEditor(
      flex,
      "city",
      wjcInput.ComboBox,
      {
        headerPath: "name",
        displayMemberPath: "name",
        itemsSource: this.cities,
        selectedValuePath: "id"
      }
    );

    // filter the custom editor
    flex.beginningEdit.addHandler(function(
      s,
      e: wjcGrid.CellRangeEventArgs
    ) {
      if(s.columns[e.col].binding === 'city') {
        setTimeout(() => {
          // get the country and city
          var cntId = e.panel.getCellData(e.row, e.col - 1, false), cityId = e.panel.getCellData(e.row, e.col, false);
          // get the current custom editor
          var cityCBHost = document.querySelector('.wj-custom-editor'), cityCB;
          // get the control
          if(cityCBHost) {
            cityCB = wjcCore.Control.getControl(cityCBHost) as wjcInput.ComboBox;
          }
          if(cityCB) {
            // filter the collectionView
            cityCB.collectionView.filter = (i => i.country === cntId);
            // set the selectedValue again because the collection is filtered
            cityCB.selectedValue = cityId;
          }

        });
      }
    });

    flex.selectionChanging.addHandler(function(
      s: wjcGrid.FlexGrid,
      e: wjcGrid.CellRangeEventArgs
    ) {
      if (e.row != s.selection.row) {
        // row is changed
        console.log("row changed");
      }
    });

    flex.lostFocus.addHandler(function(s, e) {
      console.log("lost focus");
    });

    const self = this;
    flex.rowEditStarted.addHandler(function(s, e) {
      console.log(self.data.currentPosition);
    });
  }

  private _getData() {
    // create some random data
    let data = [];
    let dt = new Date();
    for (let i = 0; i < 100; i++) {
      let cntId = this.countries[
        Math.floor(Math.random() * this.countries.length)
      ].id;
      let cities = this.cities.filter(c => c.country === cntId);
      data.push({
        id: i,
        date: new Date(dt.getFullYear(), i % 12, 25, i % 24, i % 60, i % 60),
        time: new Date(dt.getFullYear(), i % 12, 25, i % 24, i % 60, i % 60),
        country: cntId,
        city: cities[Math.floor(Math.random() * cities.length)].id,
        amount: Math.random() * 10000 - 5000,
        discount: Math.random() / 4
      });
    }

    return new wjcCore.CollectionView(data);
  }
}

@NgModule({
  imports: [WjGridModule, BrowserModule],
  declarations: [AppComponent],
  bootstrap: [AppComponent]
})
export class AppModule {}

enableProdMode();
// Bootstrap application with hash style navigation and global services.
platformBrowserDynamic().bootstrapModule(AppModule);
